import { join, resolve } from 'pathe';
import { withTrailingSlash } from 'ufo';
import { mkdirpSync, existsSync, readdirSync, copyFileSync } from 'fs-extra';

var name = "@nuxtjs/composition-api";
var version = "0.34.0";

function isFullStatic(options) {
  var _a;
  return !options.dev && !options._legacyGenerate && options.target === "static" && ((_a = options.render) == null ? void 0 : _a.ssr);
}
function isUrl(url) {
  return ["http", "//"].some((str) => url.startsWith(str));
}
function resolveRelativePath(...path) {
  return resolve(__dirname, ...path);
}
function addResolvedTemplate(template, options = {}) {
  const nuxtOptions = this.nuxt.options;
  const src = resolveRelativePath(`../runtime/templates/${template}`);
  const { dst } = this.addTemplate({
    src,
    fileName: join("composition-api", template),
    options
  });
  const templatePath = join(nuxtOptions.buildDir, dst);
  return templatePath;
}
function resolveCoreJsVersion() {
  let corejsPolyfill = this.nuxt.options.build.corejs ? String(this.nuxt.options.build.corejs) : void 0;
  try {
    if (!["2", "3"].includes(corejsPolyfill || "")) {
      const corejsPkg = this.nuxt.resolver.requireModule("core-js/package.json");
      corejsPolyfill = corejsPkg.version.slice(0, 1);
    }
  } catch (e) {
    corejsPolyfill = void 0;
  }
  return corejsPolyfill;
}
function getNuxtGlobals() {
  const nuxtOptions = this.nuxt.options;
  const globalName = nuxtOptions.globalName;
  const globalContextFactory = nuxtOptions.globals.context || ((globalName2) => `__${globalName2.toUpperCase()}__`);
  const globalNuxtFactory = nuxtOptions.globals.nuxt || ((globalName2) => `$${globalName2}`);
  const globalContext = globalContextFactory(globalName);
  const globalNuxt = globalNuxtFactory(globalName);
  return { globalContext, globalNuxt };
}

function registerBabelPlugin() {
  const nuxtOptions = this.nuxt.options;
  nuxtOptions.build.babel = nuxtOptions.build.babel || {};
  nuxtOptions.build.babel.plugins = nuxtOptions.build.babel.plugins || [];
  if (nuxtOptions.build.babel.plugins instanceof Function) {
    console.warn("Unable to automatically add Babel plugin. Make sure your custom `build.babel.plugins` returns `@nuxtjs/composition-api/dist/babel-plugin`");
  } else {
    nuxtOptions.build.babel.plugins.push(resolveRelativePath("../babel-plugin"));
  }
  const actualPresets = nuxtOptions.build.babel.presets;
  nuxtOptions.build.babel.presets = (env, [defaultPreset, defaultOptions]) => {
    const newOptions = {
      ...defaultOptions,
      jsx: {
        ...typeof defaultOptions.jsx === "object" ? defaultOptions.jsx : {},
        compositionAPI: true
      }
    };
    if (typeof actualPresets === "function") {
      return actualPresets(env, [defaultPreset, newOptions]);
    }
    return [[defaultPreset, newOptions]];
  };
}

function prepareUseStatic() {
  const nuxtOptions = this.nuxt.options;
  const staticPath = join(nuxtOptions.buildDir, "static-json");
  this.nuxt.hook("builder:prepared", () => {
    mkdirpSync(staticPath);
  });
  this.nuxt.hook("generate:route", () => {
    mkdirpSync(staticPath);
  });
  this.nuxt.hook("generate:done", async (generator) => {
    if (!existsSync(staticPath))
      return;
    const { distPath } = generator;
    readdirSync(staticPath).forEach((file) => copyFileSync(join(staticPath, file), join(distPath, file)));
  });
  return { staticPath };
}

function addGlobalsFile() {
  const nuxtOptions = this.options;
  const { staticPath } = prepareUseStatic.call(this);
  const { globalContext, globalNuxt } = getNuxtGlobals.call(this);
  const routerBase = withTrailingSlash(nuxtOptions.router.base);
  const publicPath = withTrailingSlash(nuxtOptions.build.publicPath);
  const globals = {
    isFullStatic: isFullStatic(nuxtOptions),
    staticPath,
    publicPath: isUrl(publicPath) ? publicPath : routerBase,
    globalContext,
    globalNuxt
  };
  const contents = Object.entries(globals).map(([key, value]) => `export const ${key} = ${JSON.stringify(value)}`).join("\n");
  const globalsFile = addResolvedTemplate.call(this, "globals.mjs", {
    contents
  });
  nuxtOptions.alias["@nuxtjs/composition-api/dist/runtime/globals"] = globalsFile;
}

const compositionApiModule = function compositionApiModule2() {
  var _a;
  const nuxt = this.nuxt;
  addGlobalsFile.call(this);
  const runtimeDir = resolve(__dirname, "../runtime");
  nuxt.options.build.transpile = nuxt.options.build.transpile || [];
  nuxt.options.build.transpile.push("@nuxtjs/composition-api", runtimeDir);
  const vueEntry = nuxt.options.alias.vue || (nuxt.options.dev ? this.nuxt.resolver.resolveModule("vue/dist/vue.common.dev.js") : this.nuxt.resolver.resolveModule("vue/dist/vue.runtime.esm.js"));
  const vueAliases = Object.fromEntries([
    ".common.dev",
    ".common",
    ".common.prod",
    ".esm.browser",
    ".esm.browser.min",
    ".esm",
    "",
    ".min",
    ".runtime.common.dev",
    ".runtime.common",
    ".runtime.common.prod",
    ".runtime.esm",
    ".runtime",
    ".runtime.min"
  ].flatMap((m) => [`vue/dist/vue${m}`, `vue/dist/vue${m}.js`]).map((m) => [m, vueEntry]));
  nuxt.options.alias = {
    ...vueAliases,
    ...nuxt.options.alias,
    vue: vueEntry
  };
  nuxt.options.alias["@nuxtjs/composition-api"] = nuxt.options.alias["@nuxtjs/composition-api"] || this.nuxt.resolver.resolveModule("@nuxtjs/composition-api").replace(".js", ".mjs");
  this.extendBuild((config) => {
    if (!config.module)
      return;
    config.module.rules.forEach((rule) => {
      if (rule.test instanceof RegExp && rule.test.test("index.mjs")) {
        rule.type = "javascript/auto";
      }
    });
    config.module.rules.unshift({
      test: /\.mjs$/,
      type: "javascript/auto",
      include: [/node_modules/]
    });
  });
  this.nuxt.hook("vite:extend", async (ctx) => {
    const { compositionApiPlugin } = await import('./index.mjs-vite-plugin.js');
    ctx.config.plugins.push(compositionApiPlugin());
  });
  registerBabelPlugin.call(this);
  addResolvedTemplate.call(this, "polyfill.client.mjs", {
    corejsPolyfill: resolveCoreJsVersion.call(this)
  });
  const globalPlugin = addResolvedTemplate.call(this, "plugin.mjs");
  const metaPlugin = addResolvedTemplate.call(this, "meta.mjs");
  this.nuxt.hook("modules:done", () => {
    nuxt.options.plugins.push(metaPlugin);
    nuxt.options.plugins.unshift(globalPlugin);
  });
  if (!((_a = this.nuxt.options.capi) == null ? void 0 : _a.disableMigrationWarning)) {
    this.nuxt.hook("build:done", () => {
      console.info("`Nuxt Bridge has now been released in beta.` It has full composition API support and it's strongly recommended to migrate from `@nuxtjs/composition-api`, if possible, by following the steps at `https://nuxt.com/docs/bridge/overview`. Feedback welcome at `https://github.com/nuxt-community/composition-api/discussions/585`.\n");
    });
  }
};
compositionApiModule.meta = {
  name,
  version
};

export { compositionApiModule as default };
